C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      SUBROUTINE clinput (mask, mlen, update, files, flen, nfil, axlen,
     +              goodval, badval, xoff, yoff, idbg, ierr)

C @(#)Subroutine: clinput            Author: R.A. Jansen -- Mar 19, 2013
C @(#)
C @(#)  Purpose: Parse the command-line and return two or more arguments
C @(#)    namely the name of the mask image to create or update  and the
C @(#)    name of at least one  input ASCII regions file,  as well as  a
C @(#)    set of  option=value  pairs that can be mandatory or optional,
C @(#)    depending on  whether the mask image exists  (needs updating),
C @(#)    and whether any program defaults need to be overridden.  If an
C @(#)    existing mask image needs updating, some required info such as
C @(#)    the image dimension is read from its FITS header. All returned
C @(#)    parameters are either valid and sufficient to run 'mkmask', or
C @(#)    the error status flag is raised.  Command-line option '-d' (or
C @(#)    '--debug') raises the verbosity level ('-d -d' extremely so!),
C @(#)    and the special case of command-line option '-h' (or '--help')
C @(#)    is captured.
C @(#)
C @(#)  Calling sequence:
C @(#)    CALL clinput (mask, mlen, update, files, flen, nfil, axlen,
C @(#)                  goodval, badval, xoff, yoff, idbg, ierr)  ,
C @(#)    where 'mask' is a character string and 'files' is an 8-element
C @(#)    array of character strings, both of arbitrary declared length.
C @(#)    The value of integer  'mlen'  and of the elements of 8-element
C @(#)    integer array 'flen' return the actual  (rather than declared)
C @(#)    lengths.   Integer  'nfil'  returns the number of regions file
C @(#)    names read.  Boolean 'update' will be returned true,  when the
C @(#)    specified mask image  already exists.   Integer array  'axlen'
C @(#)    will return the size  in pixels  of each image axis.  Integers
C @(#)    'goodval' and 'badval' will return either the program defaults
C @(#)    (0 and 1, respectively) or the user-specified values. Floating
C @(#)    point reals 'xoff' and 'yoff' will return the offsets in x and
C @(#)    y to apply to coordinates read from the regions files.
C @(#)    If "mkmask" was called with the optional debug switch ("-d") ,
C @(#)    diagnostic messaging will be turned on,  and the program  will
C @(#)    become  very verbose.  If it appears  that the calling routine
C @(#)    (main) should display the program's manual page,  error status
C @(#)    'ierr' will have a value of -1 upon exit;  otherwise any value
C @(#)    other than 0 indicates that an error was detected.
C @(#)
C @(#)  Note: this subroutine is specific to program "mkmask", and not a
C @(#)    generic one, and should *not* be included in any library.
C @(#)  Note: the currently supported set of  option=value  pairs is:
C @(#)    nc[ols]=#       ('#' must be an integer)
C @(#)    nr[ows]=#       ('#' must be an integer)
C @(#)    good[val]=#     ('#' will be cast to the nearest integer)
C @(#)    bad[va]=#       ('#' will be cast to the nearest integer)
C @(#)    xoff[set]=#     ('#' may be a real value)
C @(#)    yoff[set]=#     ('#' may be a real value)
C @(#)  Note: file names  (including any directory paths)  are currently
C @(#)    limited to a length of 128 chars by the declaration of 'carg'.
C @(#)    The mask file name  must be the first specified file name;  at
C @(#)    least one ASCII regions file name must follow.
C @(#)  Note: we assume,  that any integers specified  are representable
C @(#)    by a 9-character string (format 'I9').
C @(#)
C @(#)  Dependencies:     iargc and getarg (both 'libF77.a'/'libg2c.a'),
C @(#)                    several routines from my  'rjgen' and 'rjfits'
C @(#)                    libraries  (cinit, clen, cidx, creadi, creadr,
C @(#)                    error, ifmt;  rjf_open,  rjf_close,  rjfghblk,
C @(#)                    rjfghrec, rjfgkwrd, rjfgkvali).
C @(#)
C @(#)  Revision history:
C @(#)    Jan 18 2013       -- adapted from routine "clinput" in other
C @(#)    -- Feb 15 2013 [RAJ]   programs (e.g.,"fixhdr") by the author.
C @(#)    Mar 14 2013       -- merged "hdinput" with "clinput" to avoid
C @(#)    -- Mar 19 2013 [RAJ]   lengthy checks in MAIN of "mkmask"
C @(#)    Mar 21 2013 [RAJ] -- fixed bug of unterminated WHILE loop when
C @(#)                           END not found within first 2880-b block
C @(#)    May  8 2013 [RAJ] -- added 'xoff' & 'yoff' to calling sequence
C @(#)                           to support physical vs image coord.
C @(#)    Jun 28 2018 [RAJ] -- fixed bug in format statement at line 460
C @(#)------------------------------------------------------------------

      IMPLICIT NONE

C --- Learn about default devices...
      INCLUDE	'devices.inc'

C --- Learn about maximum image dimension, maximum axis lengths, maximum
C     number of header records...
      INCLUDE	'buffers.inc'

C --- Declaration of input/return variables...
      CHARACTER*(*) mask, files(8)
      REAL	xoff, yoff
      INTEGER	flen(8), axlen(MAXAX)
      INTEGER	mlen, nfil, goodval, badval, idbg, ierr
      LOGICAL	update

C --- File I/O variables and boolean switch...
      CHARACTER*2880 block
      LOGICAL	readonly

C --- Memory I/O variables...
      CHARACTER crec*80, ckey*8
      INTEGER	naxes, bpp, ival, ilen
C     -- Boolean switches and flag...
      LOGICAL	exist, lfix, lcom, lend

C --- External functions...
      INTEGER*4	iargc, narg
      INTEGER	clen, cidx, ifmt

C --- Declaration of local variables (there could be up to 18 command
C     line arguments specified!)...
      CHARACTER	ctmp*1024, carg(18)*128, cc*128, dbid*19, fmt*32, form*8
      REAL	rgood, rbad
      INTEGER	i, j, k, m, n, p

C     ------------------------------------------------------------------


C --- Return for non-zero input error status...
      IF (ierr.NE.0) RETURN

C --- Set subroutine/function identifier...
      dbid = 'DEBUG(clinput):    '

C --- Initialize the output variables...
      CALL cinit(mask)
      DO i=1,8
         CALL cinit(files(i))
         flen(i) = 0
      ENDDO
      DO i=1,MAXAX
         axlen(i) = -1
      ENDDO
      mlen    = 0
      nfil    = 0
      goodval = 0
      badval  = 1
      xoff    = 0.0
      yoff    = 0.0
      update  = .FALSE.

C --- Initialize other variables...
      naxes   = 0
      lfix    = .TRUE.
      lend    = .FALSE.

C --- Count the number of command-line arguments; capture the case of a
C     call without any arguments...
      narg = iargc()
      IF (narg.LT.1) THEN
         CALL error('clinput','MKMASK called without arguments')
         ierr = -1
         RETURN
      ENDIF

C --- Try and parse all command-line arguments and test whether any is a
C     help or debug/verbosity switch; keep others for further scrutiny..
      j = 0
      DO i=1,narg
         CALL cinit(ctmp)
         CALL getarg(i,ctmp)
         IF (ctmp(1:2).EQ.'-h'.OR.ctmp(1:3).EQ.'--h') THEN
            ierr = -1
            RETURN
         ELSE IF (ctmp(1:2).EQ.'-d'.OR.ctmp(1:3).EQ.'--d') THEN
            IF (idbg.EQ.0) THEN
               WRITE(STDOUT,'(A)') dbid//'switched ON debug mode'
            ELSE
               WRITE(STDOUT,'(A)') dbid//'raised debug verbosity level'
            ENDIF
            idbg = idbg + 1
            IF (narg.LT.2) THEN
               CALL error('clinput','missing argument(s)')
               ierr = -1
               RETURN
            ENDIF
         ELSE IF (ctmp(1:1).EQ.'-') THEN
C           We already captured all supported optional switches...
            CALL cinit(ctmp)
            ctmp = 'incorrect syntax or unsupported switch'
            CALL error('clinput',ctmp)
            ierr = -1
            RETURN
         ELSE
            j = j + 1
            IF (j.GT.18) THEN
               CALL cinit(ctmp)
               ctmp = 'more arguments than MAXARG = 18'
               CALL error('clinput',ctmp)
               ierr = -1
               RETURN
            ENDIF
            CALL cinit(carg(j))
            ilen = MIN(clen(ctmp),LEN(carg(j)))
            carg(j)(1:ilen) = ctmp(1:ilen)
         ENDIF
      ENDDO
      IF (j.LT.2) THEN
C        At a minimum, the name of the mask image to update and the name
C        of a regions file must be specified...
         CALL cinit(ctmp)
         ctmp = 'too few arguments'
         CALL error('clinput',ctmp)
         ierr = -1
         RETURN
      ENDIF


C --- When here, there was no cry for help and we have already filtered
C     out any debug/verbosity switch; the remaining command line argu-
C     ments are either  option=value  pairs or file names. Test for the
C     former first...
      DO i=1,j
         k = cidx(carg(i),'=')
         m = cidx(carg(i),'/')
         n = clen(carg(i))

         IF (k.NE.0.AND.m.EQ.0.AND.n.GE.5) THEN

C           Detected a possible option=value pair...
            IF (    (carg(i)(1:k-1).EQ.'goodval'.AND.n.GE.9)
     +          .OR.(carg(i)(1:k-1).EQ.'good'.AND.n.GE.6)) THEN
               CALL debugmsg(11,dbid,idbg,carg(i)(1:n),0,0.)

C              Try to read a numeric value from the remainder...
               CALL creadr(carg(i)(k+1:n),rgood,ierr)
               IF (ierr.NE.0) THEN
                  ctmp = 'failed to parse numeric value from option='//
     +                   'value pair: last read "good="'
                  CALL error('clinput',ctmp)
                  RETURN
               ENDIF

               goodval = NINT(rgood)
               CALL debugmsg(12,dbid,idbg,' ',goodval,0.)

            ELSE IF (    (carg(i)(1:k-1).EQ.'badval'.AND.n.GE.8)
     +               .OR.(carg(i)(1:k-1).EQ.'bad'.AND.n.GE.5)) THEN
               CALL debugmsg(11,dbid,idbg,carg(i)(1:n),0,0.)

C              Try to read a numeric value from the remainder...
               CALL creadr(carg(i)(k+1:n),rbad,ierr)
               IF (ierr.NE.0) THEN
                  ctmp = 'failed to parse numeric value from option='//
     +                   'value pair: last read "bad="'
                  CALL error('clinput',ctmp)
                  RETURN
               ENDIF

               badval = NINT(rbad)
               CALL debugmsg(13,dbid,idbg,' ',badval,0.)

            ELSE IF (    (carg(i)(1:k-1).EQ.'ncols'.AND.n.GE.7)
     +               .OR.(carg(i)(1:k-1).EQ.'ncol'.AND.n.GE.6)
     +               .OR.(carg(i)(1:k-1).EQ.'nc'.AND.n.GE.4)) THEN
               CALL debugmsg(11,dbid,idbg,carg(i)(1:n),0,0.)

C              Try to read a numeric value from the remainder...
               CALL creadi(carg(i)(k+1:n),ival,ierr)
               IF (ierr.NE.0) THEN
                  CALL cinit(cc)
                  cc(1:n-k) = carg(i)(k+1:n)
                  m = cidx(cc,'.')
                  IF (m.GT.0.AND.m.LT.clen(cc)) THEN
                     ctmp = 'value in "ncols=value" must be an integer'
                  ELSE
                     ctmp = 'failed to parse numeric value from optio'//
     +                      'n=value pair: last read "ncols="'
                  ENDIF
                  CALL error('clinput',ctmp)
                  RETURN
               ENDIF

               axlen(1) = ival
               CALL debugmsg(14,dbid,idbg,' ',axlen(1),0.)

            ELSE IF (    (carg(i)(1:k-1).EQ.'nrows'.AND.n.GE.7)
     +               .OR.(carg(i)(1:k-1).EQ.'nrow'.AND.n.GE.6)
     +               .OR.(carg(i)(1:k-1).EQ.'nr'.AND.n.GE.4)) THEN
               CALL debugmsg(11,dbid,idbg,carg(i)(1:n),0,0.)

C              Try to read a numeric value from the remainder...
               CALL creadi(carg(i)(k+1:n),ival,ierr)
               IF (ierr.NE.0) THEN
                  CALL cinit(cc)
                  cc(1:n-k) = carg(i)(k+1:n)
                  m = cidx(cc,'.')
                  IF (m.GT.0.AND.m.LT.clen(cc)) THEN
                     ctmp = 'value in "nrows=value" must be an integer'
                  ELSE
                     ctmp = 'failed to parse numeric value from optio'//
     +                      'n=value pair: last read "nrows="'
                  ENDIF
                  CALL error('clinput',ctmp)
                  RETURN
               ENDIF

               axlen(2) = ival
               CALL debugmsg(15,dbid,idbg,' ',axlen(2),0.)

            ELSE IF (    (carg(i)(1:k-1).EQ.'xoffset'.AND.n.GE.9)
     +               .OR.(carg(i)(1:k-1).EQ.'xoff'.AND.n.GE.6)) THEN
               CALL debugmsg(11,dbid,idbg,carg(i)(1:n),0,0.)

C              Try to read a numeric value from the remainder...
               CALL creadr(carg(i)(k+1:n),xoff,ierr)
               IF (ierr.NE.0) THEN
                  ctmp = 'failed to parse numeric value from option='//
     +                   'value pair: last read "xoffset="'
                  CALL error('clinput',ctmp)
                  RETURN
               ENDIF

               CALL debugmsg(16,dbid,idbg,' ',0,xoff)

            ELSE IF (    (carg(i)(1:k-1).EQ.'yoffset'.AND.n.GE.9)
     +               .OR.(carg(i)(1:k-1).EQ.'yoff'.AND.n.GE.6)) THEN
               CALL debugmsg(11,dbid,idbg,carg(i)(1:n),0,0.)

C              Try to read a numeric value from the remainder...
               CALL creadr(carg(i)(k+1:n),yoff,ierr)
               IF (ierr.NE.0) THEN
                  ctmp = 'failed to parse numeric value from option='//
     +                   'value pair: last read "yoffset="'
                  CALL error('clinput',ctmp)
                  RETURN
               ENDIF

               CALL debugmsg(17,dbid,idbg,' ',0,yoff)

            ELSE
C              Unsupported option, typo, or syntax error...
               CALL debugmsg(18,dbid,idbg,carg(i)(1:n),0,0.)
               CALL cinit(ctmp)
               ctmp = 'unsupported option, typo, or incorrect syntax'
               CALL error('clinput',ctmp)
               ierr = -1
               RETURN

            ENDIF

         ELSE
C           Not an option=value pair; assume it is a file name...
            IF (mlen.EQ.0.AND.n.GT.0) THEN
C              The first file name must be the name of the mask file...
               mask(1:n) = carg(i)(1:n)
               mlen = n
               CALL debugmsg(19,dbid,idbg,carg(i)(1:n),0,0.)
               p = MIN(n,LEN(mask))
               IF (p.LT.n) CALL debugmsg(20,dbid,idbg,' ',0,0.)

            ELSE
C              Any remaining file names must be regions file names...
               nfil = nfil + 1
               IF (nfil.GT.8) THEN
                  ctmp = 'number of regions files exceeds maximum of 8'
                  CALL error('clinput',ctmp)
                  ierr = -1
                  RETURN
               ELSE
                  files(nfil)(1:n) = carg(i)(1:n)
                  flen(nfil) = n
                  CALL debugmsg(21,dbid,idbg,carg(i)(1:n),0,0.)
                  p = MIN(n,LEN(files(nfil)))
                  IF (p.LT.n) CALL debugmsg(20,dbid,idbg,' ',0,0.)
               ENDIF

            ENDIF

         ENDIF

      ENDDO


C --- We have parsed all information provided on the command line;  now
C     test whether that provided information sufficed to run 'mkmask'.
      
C     Ignore (blank) any image section and/or extension specifiers, as
C     'mkmask' can only deal with flat FITS mask images at this time...
      p = cidx(mask,'[')
      IF (p.NE.0) THEN
         DO i = p,mlen
            mask(i:i) = ' '
         ENDDO
         mlen = p - 1
         CALL cinit(cc)
         cc(1:mlen) = mask(1:mlen)
         CALL debugmsg(22,dbid,idbg,cc(1:mlen),0,0.)
      ENDIF

C     Test whether the mask file exists already...
      INQUIRE(FILE=mask(1:mlen),EXIST=exist,FORMATTED=form)
      IF (exist) THEN
         update = .TRUE.
         CALL debugmsg(23,dbid,idbg,' ',0,0.)
      ELSE
         update = .FALSE.
         CALL debugmsg(24,dbid,idbg,' ',0,0.)
      ENDIF

C     If a mask file is to be created, axis lengths must be specified...
      IF (.NOT.update.AND.(axlen(1).EQ.-1.OR.axlen(2).EQ.-1)) THEN
         ctmp = 'cannot create a mask image without specifying length'//
     +          ' of axes'
         CALL error('clinput',ctmp)
         ierr = 1
         RETURN
      ENDIF
C     And if the mask file is to be updated, axis lengths may not be
C     modified (hence must not be specified)...
      IF (update.AND.(axlen(1).NE.-1.OR.axlen(2).NE.-1)) THEN
         ctmp = 'cannot modify length of axes in an existing mask image'
         CALL error('clinput',ctmp)
         ierr = 1
         RETURN
      ENDIF

C     If the mask image file exists, try to open it for direct read-only
C     access at the standard logical device unit reserved for input from
C     disk file, and retrieve basic info from its header...
      IF (update) THEN
         CALL debugmsg(25,dbid,idbg,mask(1:mlen),0,0.)
         readonly = .TRUE.
         CALL rjf_open (STDIFIL, mask, readonly, idbg, ierr)
         IF (ierr.NE.0) RETURN

C        Try to read the first 2880-byte block of the FITS header...
         CALL rjfghblk (STDIFIL, 1, block, idbg, ierr)
C        Capture the special case of a file of exactly 2880 bytes...
         IF (ierr.EQ.-1) ierr = 0
         IF (ierr.NE.0) RETURN

C        Try to parse the BITPIX and NAXIS* keyword values from this
C        block and verify that this is indeed a flat FITS file...
         i = 0
         DO WHILE (.NOT.lend.AND.i.LT.36)

C           Read the next 80-char record from the 2880-byte block...
            i = i + 1
            CALL rjfghrec (block, i, crec, idbg, ierr)
            IF (ierr.NE.0) RETURN
            CALL debugmsg(26,dbid,idbg,crec,i,0.)

C           Parse the keyword from the record...
            CALL rjfgkwrd (crec, ckey, j, lfix, lcom, idbg, ierr)
            IF (ierr.NE.0) RETURN

            ival = -1
            IF (ckey(1:j).EQ.'END') THEN
               CALL debugmsg(27,dbid,idbg,' ',0,0.)
               lcom = .FALSE.
               lend = .TRUE.

            ELSE IF (ckey(1:j).EQ.'BITPIX') THEN
               bpp = -1
               CALL rjfgkvali (crec, ival, idbg, ierr)
               IF (ierr.NE.0) RETURN
               bpp = ival
               CALL debugmsg(28,dbid,idbg,' ',bpp,0.)

               IF (bpp.NE.8.AND.bpp.NE.16) THEN
                  p = ifmt(bpp)
                  CALL cinit(fmt)
                  WRITE(fmt,'((A),I1,(A))') '((A),I',p,')'
                  WRITE(ctmp,fmt)
     +               'mask image is not integer valued (BITPIX=',bpp
                  CALL error('clinput',ctmp(1:clen(ctmp))//')')
                  ierr = 2
               ENDIF

            ELSE IF (ckey(1:j).EQ.'NAXIS') THEN
               CALL rjfgkvali (crec, ival, idbg, ierr)
               IF (ierr.NE.0) RETURN
               naxes = ival
               CALL debugmsg(29,dbid,idbg,' ',naxes,0.)

               IF (naxes.NE.2) THEN
                  ctmp = 'mask image is not a flat 2-D FITS file'
                  CALL error('clinput',ctmp)
                  ierr = 3
               ENDIF

            ELSE IF (ckey(1:j).EQ.'NAXIS1') THEN
               CALL rjfgkvali (crec, ival, idbg, ierr)
               IF (ierr.NE.0) RETURN
               axlen(1) = ival
               CALL debugmsg(30,dbid,idbg,' ',axlen(1),0.)

            ELSE IF (ckey(1:j).EQ.'NAXIS2') THEN
               CALL rjfgkvali (crec, ival, idbg, ierr)
               IF (ierr.NE.0) RETURN
               axlen(2) = ival
               CALL debugmsg(31,dbid,idbg,' ',axlen(2),0.)

            ELSE IF (ckey(1:j).EQ.'NEXTEND') THEN
C              This appears to be a multi-extension FITS file, not the
C              required flat FITS file...
               CALL rjfgkvali (crec, ival, idbg, ierr)
               IF (ierr.EQ.0) THEN
                  CALL debugmsg(32,dbid,idbg,' ',ival,0.)
                  ctmp = 'mask image appears to be MEF, not flat FITS'
                  CALL error('clinput',ctmp)
                  ierr = 4
               ENDIF

            ELSE
C              Some other header keyword not currently of interest...
               CONTINUE
            ENDIF

C           Test whether to abort or continue...
            IF (ierr.NE.0) THEN
               k = ierr
               ierr = 0
               CALL debugmsg(33,dbid,idbg,' ',k,0.)
               CALL rjf_close (STDIFIL, idbg, ierr)
               IF (ierr.NE.0) RETURN
               ierr = k
               RETURN
            ENDIF

         ENDDO

C        Close the currently open existing mask image file...
         CALL debugmsg(33,dbid,idbg,' ',0,0.)
         CALL rjf_close (STDIFIL, idbg, ierr)
         IF (ierr.NE.0) RETURN

      ENDIF


C     Test whether at this point both axis lengths are known and are
C     within the valid range as defined in "buffers.inc" ...
      IF (axlen(1).EQ.-1) THEN
         ctmp = 'length of axis1 is unspecified (naxis1 = -1)'
      ELSE IF (axlen(2).EQ.-1) THEN
         ctmp = 'length of axis2 is unspecified (naxis2 = -1)'
      ENDIF
      IF (axlen(1).EQ.-1.OR.axlen(2).EQ.-1) THEN
         CALL error('clinput',ctmp)
         ierr = 5
      ENDIF
      IF (axlen(1).LT.MINSIZ1) THEN
         ctmp = 'buffers.inc bounds check: naxis1 < MINSIZ1'
         CALL error('clinput',ctmp)
         ierr = 6
      ENDIF
      IF (axlen(1).GT.MAXSIZ1) THEN
         ctmp = 'buffers.inc bounds check: naxis1 > MAXSIZ1'
         CALL error('clinput',ctmp)
         ierr = 6
      ENDIF
      IF (axlen(2).LT.MINSIZ2) THEN
         ctmp = 'buffers.inc bounds check: naxis2 < MINSIZ2'
         CALL error('clinput',ctmp)
         ierr = 7
      ENDIF
      IF (axlen(2).GT.MAXSIZ2) THEN
         ctmp = 'buffers.inc bounds check: naxis2 > MAXSIZ2'
         CALL error('clinput',ctmp)
         ierr = 7
      ENDIF
      IF (ierr.NE.0) RETURN


C     Test whether there is at least one regions file specified...
      IF (nfil.LT.1) THEN
         ctmp = 'at least one regions file must be specified (nfil=0)'
         CALL error('clinput',ctmp)
         ierr = 8
         RETURN
      ENDIF


C     Test whether the specified regions files exist...
      DO i = 1,nfil
         INQUIRE(FILE=files(i)(1:flen(i)),EXIST=exist,FORMATTED=form)
         IF (.NOT.exist) THEN
            ctmp = 'specified regions file "'//files(i)(1:flen(i))//
     +             '" not found'
            CALL error('clinput',ctmp)
            ierr = 9
            RETURN
         ELSE
            CALL debugmsg(34,dbid,idbg,files(i),0,0.)
         ENDIF
      ENDDO


C --- Finally, if the verbosity level is raised, return a list of
C     program parameters that are specified upon exit...
      IF (idbg.GT.0) THEN
         WRITE(STDOUT,'(A)') dbid//'upon exit:'
         CALL cinit(cc)
         cc(1:mlen) = mask(1:mlen)
         p = ifmt(mlen)
         WRITE(fmt,'((A),I1,(A))') '((A),I',p,',(A),L1,(A))'
         WRITE(STDOUT,fmt) dbid//'  mask="'//cc(1:mlen)//
     +       '" (mlen=',mlen,', update=',update,')'
         DO i=1,nfil
            CALL cinit(cc)
            cc(1:flen(i)) = files(i)(1:flen(i))
            p = ifmt(flen(i))
            WRITE(fmt,'((A),I1,(A))') '((A),I1,(A),I1,(A),I',p,',(A))'
            WRITE(STDOUT,fmt) dbid//'  regfil[',i,']="'//cc(1:flen(i))//
     +          '" (flen[',i,']=',flen(i),')'
         ENDDO
         DO i=1,MAXAX
            p = ifmt(axlen(i))
            CALL cinit(fmt)
            WRITE(fmt,'((A),I1,(A))') '((A),I1,(A),I',p,')'
            WRITE(STDOUT,fmt) dbid//'  axlen[',i,']=',axlen(i)
         ENDDO

         i = ifmt(goodval)
         p = ifmt(badval)
         m = ifmt(bpp)
         WRITE(fmt,'((A),I1,(A),I1,(A),I1,(A))')
     +             '((A),I',i,',(A),I',p,',(A),I',m,')'
         WRITE(STDOUT,fmt) dbid//'  goodval=',goodval,', badval=',
     +             badval,', bitpix=',bpp

         WRITE(STDOUT,'((A),I1,(A),I1)') dbid//'  idbg=',idbg,
     +       ', ierr=',ierr
      ENDIF


      RETURN
      END


