C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      SUBROUTINE maskellipse (ityp, regstr, pix, ncol, nrow, mval, ierr)

C @(#)Subroutine: maskellipse         Author: R.A. Jansen -- Apr 12 2013
C @(#)
C @(#)  Purpose: parse a previously verified region descriptor string in
C @(#)    ASCII pixel list format when its type matches that of a region
C @(#)    encompassing pixels within  an elliptical region  of arbitrary
C @(#)    ellipticity and orientation (ie., 'ityp'=4) or exit otherwise,
C @(#)    and update the input pixel array  by adding the specified mask
C @(#)    value if non-zero,  or setting the specified pixel region to 0
C @(#)    otherwise.  Any out-of-bounds pixels that are part of the spe-
C @(#)    cified pixel region will be silently ignored.  If any error is
C @(#)    detected, then error status 'ierr' will be non-zero upon exit.
C @(#)  
C @(#)  Calling Sequence:
C @(#)    CALL maskellipse(ityp, regstr,  pix, ncol, nrow,  mval, ierr),
C @(#)    where 'ityp' is an integer, and 'regstr' is a character string
C @(#)    of arbitrary declared length.   A valid descriptor string con-
C @(#)    sists of  five  SPACE, TAB, or COMMA-separated  numeric values
C @(#)    in the order 'x0 y0 da db theta', where position angle 'theta'
C @(#)    is in degrees measured counter-clockwise from the positive "x"
C @(#)    axis,  while  'da' and 'db'  are the semi-major and semi-minor
C @(#)    axis size of the ellipse  measured in pixels.  'pix' passes an
C @(#)    integer valued array of size 'ncol'x'nrow' elements (with both
C @(#)    'ncol' and 'nrow' integers). 'mval' and 'ierr' are integers.
C @(#)
C @(#)  Dependencies: several routines from my  'rjgen'  library:  clen,
C @(#)    cinit, creadr, error, parse
C @(#)
C @(#)  Revision history:
C @(#)    Apr 12 2013 [RAJ] -- Initial version of this routine.
C @(#)    May  2 2013 [RAJ] -- Fixed bug in interpretation of 'theta'
C @(#) -----------------------------------------------------------------

      IMPLICIT NONE

C --- Declaration of input/return variables...
      CHARACTER*(*) regstr
      INTEGER	nrow, ncol, pix(nrow*ncol)
      INTEGER	ityp, mval, ierr

C --- External functions...
      INTEGER	clen

C --- Other program variables...
      CHARACTER delim*1, cpar*80
      REAL	rval, xcen, ycen, da, db, theta, x, y, v, w, dd
      INTEGER	lpar, k, m, i, j

C     ------------------------------------------------------------------

C --- Return for non-zero input error status...
      IF (ierr.NE.0) RETURN

C --- Return for a region descriptor string of the wrong type...
      IF (ityp.NE.4) RETURN


C --- Parse the five numeric values from the region descriptor string...
      delim = CHAR(32)
      k     = 0
      m     = clen(regstr)
      DO WHILE (m.GT.0)
         CALL cinit(cpar)
         lpar = 0
         CALL parse(regstr,delim,cpar,lpar)
         IF (lpar.GT.0) THEN
            CALL creadr(cpar(1:lpar),rval,ierr)
         ELSE
            ierr = 1
         ENDIF
         IF (ierr.NE.0) THEN
            CALL error('maskellipse','parsing ELLIPSE descriptor')
            ierr = 1
            RETURN
         ENDIF

         k = k + 1
         IF (k.EQ.1) xcen  = rval
         IF (k.EQ.2) ycen  = rval
         IF (k.EQ.3) da    = rval
         IF (k.EQ.4) db    = rval
         IF (k.EQ.5) theta = rval

         m = clen(regstr)
      ENDDO

C     Test that we successfully parsed five and only five numeric values
C     from the regions descriptor string...
      IF (k.NE.5) THEN
         CALL cinit(cpar)
         cpar = 'parsing ELLIPSE descriptor: nfields != 5'
         CALL error('maskellipse',cpar)
         ierr = 2
         RETURN
      ENDIF

C     Ensure that da>0 and db>0 ...
      IF (da.LT.0.0) THEN
         CALL error('maskellipse','invalid ELLIPSE descriptor: da<0.0')
         ierr = 3
      ENDIF
      IF (db.LT.0.0) THEN
         CALL error('maskellipse','invalid ELLIPSE descriptor: db<0.0')
         ierr = 3
      ENDIF
      IF (ierr.NE.0) RETURN

C     Reduce the position angle to [0,180>, then test whether da > db;
C     if not, then interchange da and db and set theta'=(180-theta) ...
      IF (db.EQ.da) theta = 0.0
      IF (db.GT.da) THEN
         rval  = db
         db    = da
         da    = rval
         theta = 180 - theta
         theta = 90 - theta
      ENDIF
      IF (theta.GE. 360.0) theta = theta - 360
      IF (theta.GE. 180.0) theta = theta - 180
      IF (theta.LT.   0.0) theta = theta + 180
      IF (theta.LT.-180.0) theta = theta + 360
      theta = theta*(3.1415926536/180.0)

C     ------------------------------------------------------------------
C     We use the following method to determine whether a pixel (i,j) is
C     part of the elliptical region:
C     [0] we restrict ourselves to testing pixels in a square region of
C         size 2*(da+1) x 2*(da+1) pixels, centered on (xcen,ycen), with
C         the further restriction that those pixels are within bounds;
C     [1] We apply a coordinate transformation (x,y)->(v,w) aligned with
C         the cardinal axes of the ellipse and test whether the center
C         of the pixel (i,j) being tested lies within the ellipse.
C     We will update with the mask value all pixels (i,j) that are both
C     within bounds and encompassed within the specified ellipse region.
C     ------------------------------------------------------------------

C --- Update the pixel array if the specified pixel is within bounds...
      IF (mval.EQ.0) THEN
         DO j = MAX(1,NINT(ycen-da-2)),MIN(nrow,NINT(ycen+da+2)),1
            DO i = MAX(1,NINT(xcen-da-2)),MIN(ncol,NINT(xcen+da+2)),1
               x = 1.0*i - xcen
               y = 1.0*j - ycen
               v = x*COS(theta) + y*SIN(theta)
               w = y*COS(theta) - x*SIN(theta)
               dd = SQRT( (v*v)/(da*da) + (w*w)/(db*db) )
               IF (dd.LT.1.0) THEN
C                 Pixel center falls within the ellipse...
                  pix(i+(j-1)*ncol) = 0
               ENDIF
            ENDDO
         ENDDO
      ELSE
         DO j = MAX(1,NINT(ycen-da-2)),MIN(nrow,NINT(ycen+da+2)),1
            DO i = MAX(1,NINT(xcen-da-2)),MIN(ncol,NINT(xcen+da+2)),1
               x = 1.0*i - xcen
               y = 1.0*j - ycen
               v = x*COS(theta) + y*SIN(theta)
               w = y*COS(theta) - x*SIN(theta)
               dd = SQRT( (v*v)/(da*da) + (w*w)/(db*db) )
               IF (dd.LT.1.0) THEN
C                 Pixel center falls within the ellipse...
                  pix(i+(j-1)*ncol) = pix(i+(j-1)*ncol) + mval
               ENDIF
            ENDDO
         ENDDO
      ENDIF


      RETURN
      END

