C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      SUBROUTINE maskpolygon (ityp, regstr, pix, ncol, nrow, mval, ierr)

C @(#)Subroutine: maskpolygon         Author: R.A. Jansen -- Apr 17 2013
C @(#)
C @(#)  Purpose: parse a previously verified region descriptor string in
C @(#)    ASCII pixel list format when its type matches that of an arbi-
C @(#)    trary polygon (i.e., 'ityp'=5)  or exit otherwise,  and update
C @(#)    the  input pixel array  by adding the specified mask value  if
C @(#)    nonzero, or setting the specified pixel region to 0 otherwise.
C @(#)    Any out-of-bounds pixels  that are part of the specified pixel
C @(#)    region will be silently ignored. Should any error condition be
C @(#)    detected, then error status 'ierr' will be non-zero upon exit.
C @(#)  
C @(#)  Calling Sequence:
C @(#)    CALL maskpolygon (ityp, regstr, pix, ncol, nrow, mval, ierr) ,
C @(#)    where 'ityp' is an integer, and 'regstr' is a character string
C @(#)    of arbitrary declared length.   A valid descriptor string con-
C @(#)    sists of '2N' SPACE, TAB, or COMMA-separated numeric values in
C @(#)    the order 'x1 y1 x2 y2 .. xN yN', for all 3 <= N <= 256, where
C @(#)    each (xi,yi) pair  corresponds  to  a vertex  of the  polygon.
C @(#)    'pix'  passes  an integer valued array  of size  'ncol'x'nrow'
C @(#)    pixels  (with  'ncol' and 'nrow'  integers).  Both  'mval' and
C @(#)    'ierr' are integers.
C @(#)
C @(#)  Note: previous verification likely only consisted of a test that
C @(#)    the region descriptor string contains an even number ('2N') of
C @(#)    numeric values and that 3<=N<=256  (minimum number of vertices
C @(#)    that corresponds to  an arbitrarilly oriented triangle  to the
C @(#)    limit on the buffer sizes in the present routine).
C @(#)
C @(#)  Dependencies: several routines from my  'rjgen'  library:  clen,
C @(#)    cinit, creadr, error, parse
C @(#)
C @(#)  Revision history:
C @(#)    Apr 18 2013 [RAJ] -- Initial version of this routine.
C @(#)    May  4 2013 [RAJ] -- Major revision of original (flawed) logic
C @(#)    May  7 2013 [RAJ] -- Bug fix of erroneous masking when there is
C @(#)                         only one pixel pre-masked along a row; and
C @(#)                         there's still an issue with vertical inden
C @(#)                         tations not being properly masked...
C @(#) -----------------------------------------------------------------

      IMPLICIT NONE

C --- Declaration of input/return variables...
      CHARACTER*(*) regstr
      INTEGER	nrow, ncol, pix(nrow*ncol), tmp(nrow*ncol)
      INTEGER	ityp, mval, ierr

C --- External functions...
      INTEGER	clen

C --- Other program variables...
      CHARACTER delim*1, cpar*80
      REAL	rx(256), ry(256), a(256), b(256), dx(256), dy(256)
      REAL	rval, xmin, xmax, ymin, ymax, xi, y1, y2
      INTEGER   lpar, imin, imax, jmin, jmax
      INTEGER	k, m, n, p, q, i, j
      LOGICAL   outofbounds, lmask

C     ------------------------------------------------------------------

C --- Return for non-zero input error status...
      IF (ierr.NE.0) RETURN

C --- Return for a region descriptor string of the wrong type...
      IF (ityp.NE.5) RETURN


C --- Parse N pairs of numeric values from the region descriptor string
      delim = CHAR(32)
      k = clen(regstr)
      m = 0
      n = 0

C     write(*,*)'DEBUG(maskpolygon): [entering] clen(regstr)=',k

      DO WHILE (k.GT.0)
         CALL cinit(cpar)
         lpar = 0
         CALL parse(regstr,delim,cpar,lpar)
         IF (lpar.GT.0) THEN
            CALL creadr(cpar(1:lpar),rval,ierr)
         ELSE
            ierr = 1
         ENDIF
         IF (ierr.NE.0) THEN
            CALL error('maskpolygon','parsing POLYGON descriptor')
            ierr = 1
            RETURN
         ENDIF

         m = m + 1
         IF (MOD(m,2).EQ.1) n = n + 1
         IF (n.GT.256) THEN
            CALL cinit(cpar)
            cpar = 'number of vertices exceeds maximum of 256'
            CALL error('maskpolygon',cpar)
            ierr = 2
            RETURN
         ENDIF
         IF (MOD(m,2).EQ.1) rx(n) = rval
         IF (MOD(m,2).EQ.0) ry(n) = rval
C        IF (MOD(m,2).EQ.0) write(*,*)'DEBUG(maskpolygon): '//
C    +       'new vertex (',rx(n),',',ry(n),')'

         k = clen(regstr)
      ENDDO

C     Test that we successfully parsed 2N numeric values from the
C     regions descriptor string and that N >=3 ...
      IF (MOD(m,2).NE.0) THEN
         CALL cinit(cpar)
         cpar = 'parsing POLYGON descriptor: odd number of values!'
         CALL error('maskpolygon',cpar)
         ierr = 3
         RETURN
      ENDIF
      IF (n.LT.3) THEN
         CALL cinit(cpar)
         cpar = 'number of vertices < 3: region is not a polygon!'
         CALL error('maskpolygon',cpar)
         ierr = 4
         RETURN
      ENDIF

C     write(*,*)'DEBUG(maskpolygon): read ',n,' vertices'

C     Test whether any pixels are within bounds; an out-of-bounds region
C     or partially out-of-bounds region does not qualify as an error...
      outofbounds = .TRUE.
C     write(*,*)'DEBUG(maskpolygon):  ncol=',ncol,', nrow=',nrow
      DO p=1,n
         IF (     NINT(rx(p)).GE.1.AND.NINT(rx(p)).LE.ncol
     +       .AND.NINT(ry(p)).GE.1.AND.NINT(ry(p)).LE.nrow )
     +       outofbounds = .FALSE.

C        write(*,*)'DEBUG(maskpolygon):  '//
C    +        'vertex (',rx(p),',',ry(p),'): outofbounds = ',outofbounds

      ENDDO
      IF (outofbounds) THEN
C        The specified polygon lies entirely out of bounds; so we have
C        nothing to do...
C        write(*,*)'DEBUG(maskpolygon): specified polygon out of bounds'
         RETURN
      ENDIF

C     ------------------------------------------------------------------
C     We use the following method to determine whether a pixel (i,j)
C     being tested lies within the region enclosed by the polygon:
C     [0] determine the minimum and maximum vertex x and y values:
C             xmin = MIN(x[p]) ; xmax = MAX(x[p])    with p=1,2,..,N
C             ymin = MIN(y[p]) ; ymax = MAX(y[p])
C     [1] starting from the first vertex, compute the line equations for
C         each of the N line segments that define the polygon boundary:
C             slopes:        a[p] = (y[p+1] - y[p])/(x[p+1] - x[p])
C             y-intercepts:  b[p] = y[p] - x[p]*a[p]
C             delta-x:       dx[p] = (x[p+1] - x[p])
C             delta-y:       dy[p] = (y[p+1] - y[p])
C         where the final end point (x[N+1],y[N+1]) = (x[1],y[1]) again.
C     [2] looping over the line segments, and going line-by-line through
C         a temporary mask array representing the region defined by the
C         minimum and maximum x and y values of the vertices (restricted
C         to in-bounds pixels in the mask), mask the pixel nearest the
C         x-intercept of the current segment, found by equating the two
C         line equations:    a[p]*x + b[p] = j  -->  x = (j - b[p])/a[p]
C     [3] looping line by line over this temporary array, search for the
C         first masked pixel (if any) in that line, and continue masking
C         all subsequent pixels until another masked pixel is found, or
C         the end of that line is reached. If the former, search for the
C         next masked pixel and so on.  I.e., we will set and unset a
C         boolean flag each time we find an already masked pixel.  This
C         will allow polygons of arbitrary shape and complexity (with a
C         caveat for deep vertical indentations in the polygon shape!)
C     [4] All pixels masked in the temporary array are pixels that need
C         to be masked in the output mask array.
C     ------------------------------------------------------------------

C --- Compute the minimum and maximum x and y values of the vertices,
C     and compute the slopes, y-intercepts, delta-x and delta-y values
C     for each line segment defining the polygon boundary...
      xmin =  1.0E6
      xmax = -1.0E6
      ymin =  1.0E6
      ymax = -1.0E6
      DO p=1,n
         IF (rx(p).LE.xmin) xmin = rx(p)
         IF (rx(p).GE.xmax) xmax = rx(p)
         IF (ry(p).LE.ymin) ymin = ry(p)
         IF (ry(p).GE.ymax) ymax = ry(p)
         IF (p.EQ.n) THEN
            dx(p) = (rx(1) - rx(p))
            dy(p) = (ry(1) - ry(p))
         ELSE
            dx(p) = (rx(p+1) - rx(p))
            dy(p) = (ry(p+1) - ry(p))
         ENDIF
         a(p) = dy(p)/dx(p)
         b(p) = ry(p) - rx(p)*a(p)

C        write(*,*)'DEBUG(maskpolygon): seg#',p,': dx=',dx(p),
C    +             ', dy=',dy(p),'; a=',a(p),', b=',b(p)

      ENDDO

C --- Initialize the relevant portion of the temporary array to zero...
      imin = MAX(1,NINT(xmin))
      imax = MIN(ncol,NINT(xmax))
      jmin = MAX(1,NINT(ymin))
      jmax = MIN(nrow,NINT(ymax))
      DO j = jmin,jmax,1
         DO i = imin,imax,1
            tmp(i+(j-1)*ncol) = 0
         ENDDO
      ENDDO

C --- Loop over each line segment defining the polygon boundary and
C     line-by-line, mask the pixel within bounds that is nearest the
C     x-intercept of that boundary segment...
C     write(*,*)'DEBUG(maskpolygon):  finding boundary pixels...'
      DO p=1,n

C        Determine relevant portion of the array and the sign of the
C        loop increment...
         y1 = ry(p)
         IF (p.EQ.n) THEN
            y2 = ry(1)
         ELSE
            y2 = ry(p+1)
         ENDIF
         k = 1
         IF (y2.LT.y1) k = -1

C        Loop over that portion line-by-line and mask the pixel nearest
C        the x-intercept of that segment...
         DO j=MAX(1,NINT(y1)),MIN(nrow,NINT(y2)),k
            IF (ABS(dx(p)).LT.1.0E-6) THEN
C              Special case of a vertical boundary...
               xi = rx(p)
            ELSE
C              Boundary with a finite slope; a(p)=0 is OK...
               xi = (j - b(p))/a(p)
            ENDIF

C           Catch the case of a partially out-of-bounds region (and
C           zero slope boundary segments)...
            i  = NINT(xi)
            IF (i.LT.1) THEN
               i = 1
            ELSE IF (i.GT.ncol) THEN
               i = ncol
            ENDIF

C           We have found a boundary (or border) pixel; mask it...
C           write(*,*)'boundary pixel (',i,',',j,'), vertex #',p
            tmp(i+(j-1)*ncol) = p
         ENDDO

      ENDDO

C --- Loop line by line over the temporary array and find pairs of boun-
C     dary pixels and mask all pixels in the output array between these
C     pairs of boundary pixels; catch the special case of single bottom
C     or top vertex boundary pixel, and of a pair of adjacent boundary
C     pixels...
      DO j = jmin,jmax,1
         lmask = .FALSE.

         DO i = imin,imax,1
            IF (tmp(i+(j-1)*ncol).GT.0) THEN

C              Found a boundary pixel; mask it in the output array...
               IF (mval.EQ.0) THEN
                  pix(i+(j-1)*ncol) = 0
               ELSE
                  pix(i+(j-1)*ncol) = pix(i+(j-1)*ncol) + mval
               ENDIF

C              Turn off masking switch if it was on, or on when it was
C              off, unless the current pixel is the only boundary pixel
C              in the current line...  [Polygons exist that will not be
C              correctly masked using the following logic! Vertical in-
C              dentations are a problem, alas]
               IF (lmask) THEN
C                 Do more boundary pixel(s) follow in current line?
                  q = 0
                  DO k=i+1,imax,1
                     IF (     tmp(k+(j-1)*ncol).GT.0
     +                   .AND.tmp((k-1)+(j-1)*ncol).EQ.0) q = q + 1
                  ENDDO
                  IF (MOD(q,2).EQ.0) lmask = .FALSE.
C                 IF (q.EQ.0) write(*,*)'no more bounds in line ',j

               ELSE
C                 Do more boundary pixel(s) follow in current line?
                  q = 0
                  DO k=i+1,imax,1
                     IF (     tmp(k+(j-1)*ncol).GT.0
     +                   .AND.tmp((k-1)+(j-1)*ncol).EQ.0) q = q + 1
                  ENDDO
                  IF (MOD(q,2).EQ.1) lmask = .TRUE.
C                 IF (q.GT.0) write(*,*) q,' more bounds in line ',j

               ENDIF

C************* produce mask of only the boundary pixels; don't fill..
C              lmask = .FALSE.
C************* end of boundary pixel mask test.
C************* highlight boundary pixels with their segment number...
C              pix(i+(j-1)*ncol) = tmp(i+(j-1)*ncol)
C************* end of highlighting.

            ELSE
C              Mask current pixel if the masking switch is turned on...
               IF (lmask) THEN
                  IF (mval.EQ.0) THEN
                     pix(i+(j-1)*ncol) = 0
                  ELSE
                     pix(i+(j-1)*ncol) = pix(i+(j-1)*ncol) + mval
                  ENDIF
               ENDIF
            ENDIF
         ENDDO
      ENDDO


      RETURN
      END

