C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      SUBROUTINE maskrectang (ityp, regstr, pix, ncol, nrow, mval, ierr)

C @(#)Subroutine: maskrectang         Author: R.A. Jansen -- Apr 10 2013
C @(#)
C @(#)  Purpose: parse a previously verified region descriptor string in
C @(#)    ASCII pixel list format when its type matches that of a region
C @(#)    encompassing pixels within a rectangular box whose orientation
C @(#)    matches the pixel grid (ie., 'ityp'=3) or exit otherwise,  and
C @(#)    update  the  input pixel array  by adding  the  specified mask
C @(#)    value if non-zero,  or setting the specified pixel region to 0
C @(#)    otherwise.  Any out-of-bounds pixels that are part of the spe-
C @(#)    cified pixel region will be silently ignored.  If any error is
C @(#)    detected, then error status 'ierr' will be non-zero upon exit.
C @(#)  
C @(#)  Calling Sequence:
C @(#)    CALL maskrectang(ityp, regstr,  pix, ncol, nrow,  mval, ierr),
C @(#)    where 'ityp' is an integer, and 'regstr' is a character string
C @(#)    of arbitrary declared length.   A valid descriptor string con-
C @(#)    sists of  four  SPACE, TAB, or COMMA-separated  numeric values
C @(#)    in the order 'x1 x2 y1 y2',  whose values  will be rounded  to
C @(#)    the nearest integer,  if they are not integers already.  'pix'
C @(#)    passes an integer valued array  of size 'ncol'x'nrow' elements
C @(#)    (with 'ncol' and 'nrow' integers).  Both 'mval' and 'ierr' are
C @(#)    integers.
C @(#)
C @(#)  Dependencies: several routines from my  'rjgen'  library:  clen,
C @(#)    cinit, creadr, error, parse
C @(#)
C @(#)  Revision history:
C @(#)    Apr 11 2013 [RAJ] -- Initial version of this routine.
C @(#) -----------------------------------------------------------------

      IMPLICIT NONE

C --- Declaration of input/return variables...
      CHARACTER*(*) regstr
      INTEGER	nrow, ncol, pix(nrow*ncol)
      INTEGER	ityp, mval, ierr

C --- External functions...
      INTEGER	clen

C --- Other program variables...
      CHARACTER delim*1, cpar*80
      REAL	rval
      INTEGER	ix1, ix2, iy1, iy2
      INTEGER	lpar, k, m, i, j

C     ------------------------------------------------------------------

C --- Return for non-zero input error status...
      IF (ierr.NE.0) RETURN

C --- Return for a region descriptor string of the wrong type...
      IF (ityp.NE.3) RETURN


C --- Parse the four numeric values from the region descriptor string...
      delim = CHAR(32)
      k     = 0
      m     = clen(regstr)

C     write(*,*)'DEBUG(maskrectang): [entering] clen(regstr)=',m,
C    +          ', mval=',mval

      DO WHILE (m.GT.0)
         CALL cinit(cpar)
         lpar = 0
         CALL parse(regstr,delim,cpar,lpar)
         IF (lpar.GT.0) THEN
            CALL creadr(cpar(1:lpar),rval,ierr)
         ELSE
            ierr = 1
         ENDIF
         IF (ierr.NE.0) THEN
            CALL error('maskrectang','parsing RECTANG descriptor')
            ierr = 1
            RETURN
         ENDIF

         k = k + 1
         IF (k.EQ.1) ix1 = NINT(rval)
         IF (k.EQ.2) ix2 = NINT(rval)
         IF (k.EQ.3) iy1 = NINT(rval)
         IF (k.EQ.4) iy2 = NINT(rval)

         m = clen(regstr)
      ENDDO

C     Test that we successfully parsed four and only four numeric values
C     from the regions descriptor string...
      IF (k.NE.4) THEN
         CALL cinit(cpar)
         cpar = 'parsing RECTANG descriptor: nfields != 4'
         CALL error('maskrectang',cpar)
         ierr = 2
         RETURN
      ENDIF

C     write(*,*)'DEBUG(maskrectang): read 4 numeric values'

C     Ensure that x1<=x2 and y1<=y2 ...
      IF (ix1.GT.ix2) THEN
         k = ix1
         ix1 = ix2
         ix2 = k
      ENDIF
      IF (iy1.GT.iy2) THEN
         k = iy1
         iy1 = iy2
         iy2 = k
      ENDIF

C     write(*,*)'DEBUG(maskrectang): x1 x2 y1 y2 = ',ix1,ix2,iy1,iy2

C     Test whether any pixels are within bound; an out-of-bounds region
C     or partially out-of-bounds region does not qualify as an error...
      IF (    (ix1.LT.1.AND.ix2.LT.1)
     +    .OR.(ix1.GT.ncol.AND.ix2.GT.ncol)
     +    .OR.(iy1.LT.1.AND.iy2.LT.1)
     +    .OR.(iy1.GT.nrow.AND.iy2.GT.nrow) ) THEN
C        The specified pixel region lies entirely out of bounds; so we
C        have nothing to do...
C        write(*,*)'DEBUG(maskrectang): specified box out of bounds'
         RETURN
      ENDIF

C     ------------------------------------------------------------------
C     We will update with the mask value all pixels (i,j) that are both
C     within bounds and encompassed in x1 <= i <= x2 and y1 <= j <= y2
C     ------------------------------------------------------------------

C --- Update the pixel array if the specified pixel is within bounds...
C     write(*,*)'DEBUG(maskrectang):  updating pixel array...'
      IF (mval.EQ.0) THEN
         DO j = MAX(1,iy1),MIN(nrow,iy2),1
            DO i = MAX(1,ix1),MIN(ncol,ix2),1
               pix(i+(j-1)*ncol) = 0
            ENDDO
         ENDDO
      ELSE
         DO j = MAX(1,iy1),MIN(nrow,iy2),1
            DO i = MAX(1,ix1),MIN(ncol,ix2),1
               pix(i+(j-1)*ncol) = pix(i+(j-1)*ncol) + mval
            ENDDO
         ENDDO
      ENDIF


      RETURN
      END

