C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      SUBROUTINE nparse (cstr, npar, pars, lint, idbg, ierr)

C @(#)Subroutine: nparse              Author: R.A. Jansen -- Apr 22 2013
C @(#)
C @(#)  Purpose:  parse character string 'cstr', known to contain 'npar'
C @(#)    numeric values, into fields (interpreting SPACE, COMMA and TAB
C @(#)    as field-separating characters),  and return real-valued array
C @(#)    'pars'  with the numeric values  successfully read  from these
C @(#)    fields. If switch 'lint' is .TRUE. on input, round all numeric
C @(#)    values  to the floating point representation  of their nearest
C @(#)    integer value.  When the number of numeric values successfully
C @(#)    read from 'cstr' does not match 'npar',  an error message will
C @(#)    be issued.  If verbosity level 'idbg' < 0, the non-fatal error
C @(#)    of reading a smaller number of values than specified,  will be
C @(#)    be suppressed (useful when the number of numeric values is not
C @(#)    known when this routine is called, but when the maximum number
C @(#)    can be specified).  In this case,  the value of 'idbg' will be
C @(#)    reset to (-1-idbg) upon exit. When verbosity level 'idbg' > 0,
C @(#)    additional message  may be printed.  If any error condition is
C @(#)    detected (and not suppressed), error status 'ierr' will be re-
C @(#)    turned non-zero upon exit.
C @(#)
C @(#)  Calling sequence: CALL nparse (cstr,npar,pars,lint,idbg,ierr)  ,
C @(#)    with  'cstr'  a character string of arbitrary declared length,
C @(#)    'npar' an integer,  'pars'  a real-valued array with arbitrary
C @(#)    declared size,  'lint' a boolean switch,  and both 'idbg'  and
C @(#)    'ierr' integers.   Note, that 'cstr' will be returned empty on
C @(#)    successful completion of this routine.
C @(#)
C @(#)  Dependencies: several routines from my  'rjgen'  library:  clen,
C @(#)    cinit, creadr, error, parse
C @(#)
C @(#)  Revision history:
C @(#)    Apr 22, 2013 [RAJ] -- Initial version of this routine.
C @(#)    Apr 23, 2013 [RAJ] -- Implemented suppressing non-fatal error,
C @(#)                          of mpar=/=npar, if idbg<0 on invocation.
C @(#)    May  1, 2013 [RAJ] -- Added some debugging messages...
C @(#)------------------------------------------------------------------

      IMPLICIT NONE

C --- Learn about default devices...
      INCLUDE	'devices.inc'

C --- Declaration of input/return variables...
      CHARACTER*(*) cstr
      INTEGER	npar, idbg, ierr
      REAL	pars(*)
      LOGICAL	lint

C --- External functions...
      INTEGER	clen

C --- Declaration of local variables...
      CHARACTER	cpar*80, dbid*19, delim*1
      REAL	rval
      INTEGER	lpar, mlen, mpar

C     ------------------------------------------------------------------


C --- Return for non-zero input error status...
      IF (ierr.NE.0) RETURN

C --- Set subroutine/function identifier...
      dbid = 'DEBUG(nparse):     '

C --- Initialize output array and field counter...
      DO mpar=1,npar
         pars(mpar) = 0.0
      ENDDO
      mpar = 0

C --- Parse one field at a time from the input string and try to read a
C     numeric value from that field...
      delim = CHAR(32)
      mlen  = clen(cstr)

      IF (idbg.GT.1.OR.idbg.LT.-2) WRITE(STDOUT,'(A25,I4,A7,I3,A7,I3)')
     +      dbid//' mlen=',mlen,', idbg=',idbg,', npar=',npar

      DO WHILE (mlen.GT.0)
C        IF (idbg.GT.1.OR.idbg.LT.-2) WRITE(STDOUT,*) 
C    +      dbid//' entered or continuing DO WHILE loop...'

         CALL cinit(cpar)
         lpar  = 0
C         WRITE(*,*) dbid//'[before parse] cstr="'//cstr(1:80)//'"'
         CALL parse(cstr,delim,cpar,lpar)
C         WRITE(*,*) dbid//'[after parse] cstr="'//cstr(1:80)//'"'

         IF (lpar.GT.0) THEN
C           WRITE(*,*) dbid//'[before creadr] cpar="'//cpar(1:lpar)//
C    +        '" lpar=',lpar
            CALL creadr(cpar(1:lpar),rval,ierr)
         ELSE
            ierr = 1
         ENDIF
         IF (idbg.GT.1.OR.idbg.LT.-2) THEN
            WRITE(STDOUT,'(A34,I3,A1,F8.2,A8,I1,A1)') dbid//
     +           ' region field #',(mpar+1),'=',rval,'  (ierr=',ierr,')'
         ENDIF

         IF (ierr.NE.0) THEN
            CALL error('nparse','parsing region descriptor string')
            ierr = 1
            RETURN
         ENDIF

         mpar = mpar + 1
         IF (lint) THEN
C           write(*,*)dbid//' lint=.TRUE. [rounding to nearest integer]'
C           write(*,*)dbid//' [if lint] mpar=',mpar
            pars(mpar) = FLOAT(NINT(rval))
         ELSE
            pars(mpar) = rval
         ENDIF

         mlen = clen(cstr)
C        WRITE(*,*) dbid//'[continue loop?] mlen=',mlen

      ENDDO

      IF (idbg.GT.1.OR.idbg.LT.-2) THEN
            WRITE(STDOUT,'(A38,I2,A7,I3,A7,I3)') dbid//
     +          '[after loop]  mlen=',mlen,', mpar=',mpar,', npar=',npar
      ENDIF

C --- Test that we successfully parsed the required number of numeric
C     values from the descriptor string (if idbg>=0) ...
      IF (mpar.NE.npar.AND.idbg.GE.0) THEN
         CALL cinit(cpar)
         WRITE(cpar,'(A37,I3)')'parsing descriptor string: nfields !=',
     +                          npar
         CALL error('nparse',cpar)
         ierr = 2
      ENDIF
C     Return the actually read number of numeric values and reset the
C     verbosity level (if idbg<0) ...
      IF (idbg.LT.0) THEN
         npar = mpar
         idbg = -1 - idbg
      ENDIF


      RETURN
      END


