C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      SUBROUTINE rcfitsi(filename,pix,ncol,nrow,idbg,ierr)

C @(#)Subroutine: rcfitsi            Author: R.A. Jansen -- Mar 20, 2013
C @(#)
C @(#)  Purpose:  Read a FITS image or the specified extension in a FITS
C @(#)    file into 2-D pixel buffer 'pix' and return the length of each
C @(#)    image axis  (for that extension).  The image is read in pieces
C @(#)    of MAXSIZ1 pixels at a time, rather than all at once.  One may
C @(#)    specify an image extension as part of the file name by append-
C @(#)    ing its number within square bracktes (e.g., "image.fits[2]").
C @(#)    This routine is also able to read  IRAF  OIF-format  (old IRAF
C @(#)    format ".imh") images. If verbosity level 'idbg'>0, additional
C @(#)    messages  may be printed.  If any error condition is detected,
C @(#)    error status 'ierr' will be non-zero upon exit.
C @(#)
C @(#)  Calling sequence: CALL rcfitsi(filename,pix,ncol,nrow,idbg,ierr)
C @(#)    where  'filename'  is a character string of arbitrary declared
C @(#)    length,  'pix' is an integer valued array  with size MAXSIZ1 x
C @(#)    MAXSIZ2 pixels. Integers  'ncol' and 'nrow'  return the actual
C @(#)    image size.  Both 'idbg' and 'ierr' are integers.
C @(#)
C @(#)  Dependencies: several routines from the CFITSIO v3.0 I/O library
C @(#)    ('libcfitsio.a'; W.D. Pence), called using their f77 wrappers;
C @(#)    several routines from my 'rjgen' library:  cidx, cinit, cldx,
C @(#)            clen, ifmt, error, wsizstr
C @(#)
C @(#)  Note: compilation requires linking against both 'libcfitsio' and
C @(#)    against 'libm' (the standard math library).
C @(#)
C @(#)  Revision history:
C @(#)    Mar 20 2013 [RAJ] -- Major revision of original code extracted
C @(#)    --Mar 21 2013        from 'elphot_alt.f' [Jun 2005];  expanded
C @(#)                         documentation.
C @(#)    Mar 27 2013 [RAJ] -- Added support for DBLE and REAL images.
C @(#)    Apr 30 2013 [RAJ] -- Added access to FITSIO error messages for
C @(#)                         high verbosity levels.
C @(#) -----------------------------------------------------------------

      IMPLICIT NONE

C --- Learn about default devices...
      INCLUDE	'devices.inc'

C --- Learn about maximum image dimension, maximum axis lengths...
      INCLUDE	'buffers.inc'

C --- Declaration of input/return variables...
      CHARACTER*(*) filename
      INTEGER	pix(MAXSIZ2*MAXSIZ1), ncol, nrow, idbg, ierr

C --- File I/O variables and boolean switch...
      INTEGER	readonly

C --- Memory I/O variables...
      REAL*8	tmpdpx(MAXSIZ1)
      REAL	tmprpx(MAXSIZ1), bzero
      INTEGER	tmppix(MAXSIZ1), ihdu, nhdu, htyp, bpp, naxes, axlen(2)
      INTEGER*2 tmpipx(MAXSIZ1)
      INTEGER*1 tmpbpx(MAXSIZ1)
      LOGICAL	anynull

C --- External functions...
      INTEGER	cidx, cldx, clen, ifmt

C --- Other program variables and loop counters...
      CHARACTER	ctmp*80, cext*8, dbid*19, fmt*32
      INTEGER	pix1, i, j, k, m, n

C     ------------------------------------------------------------------


C --- Return for non-zero input error status...
      IF (ierr.NE.0) RETURN

C --- Set subroutine/function identifier...
      dbid = 'DEBUG(rcfitsi):    '

C --- Initialize output and local variables...
      ncol = -1
      nrow = -1
      CALL cinit(ctmp)
      CALL cinit(cext)
      CALL cinit(fmt)


C --- Check whether the image name includes an extension specifier...
      i = cidx(filename,'[')
      j = cldx(filename,']')
      IF (i.GT.0.AND.j.GT.i) THEN
C        The filename includes an extension specifier...
         k = j - i - 1
         m = j - i + 1
         ctmp(1:m) = filename(i:j)
         IF (k.GT.3) THEN
            WRITE(STDOUT,'(A)') dbid//' cext='//ctmp(1:m)
            CALL error('rcfitsi','malformatted extension specifier')
            ierr = 1
            RETURN
         ENDIF
         IF(idbg.GT.1) WRITE(STDOUT,'(A)') dbid//' cext='//ctmp(1:m)
         cext = '  '//filename(i+1:j-1)//' '
         READ (cext,'(I3)',ERR=9) ihdu
      ELSE
         IF(idbg.GT.1) WRITE(STDOUT,'(A)') dbid//' cext=""'
         ihdu = 0
      ENDIF
      IF (idbg.GT.0) THEN
         k = ifmt(ihdu)
         WRITE(fmt,'((A),I1,A1)') '((A),I',k,')'
         WRITE(STDOUT,fmt) dbid//' requested HDU = ',ihdu
      ENDIF


C --- Open input FITS or OIF image read-only at the standard logical
C     device unit for input from disk file, and move to the specified
C     HDU (whether primary or extension)...
      readonly = 0
      CALL ftnopn (STDIFIL, filename, readonly, ierr)
      IF (ierr.NE.0.AND.idbg.GT.1) CALL ftrprt ('STDOUT', ierr)
      IF (ierr.NE.0) THEN
         ctmp = 'opening "'//filename(1:clen(filename))//'"'
         CALL error('rcfitsi',ctmp)
         ierr = 2
         RETURN
      ENDIF

C --- Check that the current HDU is the requested one...
      CALL ftghdn (STDIFIL, nhdu)
      IF (nhdu.NE.(ihdu+1)) THEN
         IF (idbg.GT.0) THEN
            k = ifmt(nhdu)
            WRITE(fmt,'((A),I1,A1)') '((A),I',k,')'
            WRITE(STDOUT,fmt) dbid//' current HDU = ',nhdu
         ENDIF
         CALL error('rcfitsi','failed to open requested HDU')
         ierr = 3
         RETURN
      ENDIF

C --- Check that the current HDU has type 'IMAGE' (htyp=0)...
      CALL ftghdt (STDIFIL, htyp, ierr)
      IF (ierr.NE.0.AND.idbg.GT.1) CALL ftrprt ('STDOUT', ierr)
      IF (ierr.NE.0.OR.htyp.NE.0) THEN
         IF (idbg.GT.0) THEN
            k = ifmt(htyp)
            m = ifmt(ierr)
            WRITE(fmt,'((A),I1,A1)') '((A),I',k,',(A),I',m,')'
            WRITE(STDOUT,fmt) dbid//'   htyp = ',htyp,', ierr =',ierr
         ENDIF
         CALL error('rcfitsi','current HDU not of type "IMAGE"')
         ierr = 4
         RETURN
      ENDIF


C --- Determine the size of the image and the pixel data type...
      CALL ftgipr (STDIFIL, 2, bpp, naxes, axlen, ierr)
      IF (ierr.NE.0.AND.idbg.GT.1) CALL ftrprt ('STDOUT', ierr)
      IF (ierr.NE.0) THEN
         CALL error('rcfitsi','failed to read image size and data type')
         ierr = 5
         RETURN
      ENDIF
      ncol = axlen(1)
      nrow = axlen(2)
      IF (naxes.NE.2) THEN
         ctmp = filename(1:clen(filename))//' is not a 2-D image'
         CALL error('rcfitsi',ctmp)
         ierr = 6
      ENDIF
      IF (ncol.LT.MINSIZ1) THEN
         ctmp = 'buffers.inc bounds check: ncol < MINSIZ1'
         CALL error('rcfitsi',ctmp)
         ierr = 7
      ENDIF
      IF (ncol.GT.MAXSIZ1) THEN
         ctmp = 'buffers.inc bounds check: ncol > MAXSIZ1'
         CALL error('rcfitsi',ctmp)
         ierr = 7
      ENDIF
      IF (nrow.LT.MINSIZ2) THEN
         ctmp = 'buffers.inc bounds check: nrow < MINSIZ2'
         CALL error('rcfitsi',ctmp)
         ierr = 8
      ENDIF
      IF (nrow.GT.MAXSIZ2) THEN
         ctmp = 'buffers.inc bounds check: nrow > MAXSIZ2'
         CALL error('rcfitsi',ctmp)
         ierr = 8
      ENDIF
      IF (ierr.NE.0) RETURN

      CALL cinit(cext)
      CALL ftgkye (STDIFIL, 'BZERO', bzero, cext, ierr)
      IF (ierr.NE.0.AND.idbg.GT.1) CALL ftrprt ('STDOUT', ierr)
      IF (ierr.NE.0) THEN
         bzero = 0.0
         ierr  = 0
      ENDIF
      IF (bpp.EQ.-64) cext = '[dble]'
      IF (bpp.EQ.-32) cext = '[real]'
      IF (bpp.EQ.32)  cext = '[long]'
      IF (bpp.EQ.16.AND.bzero.EQ.32768.0) cext = '[ushort]'
      IF (bpp.EQ.16.AND.bzero.NE.32768.0) cext = '[short]'
      IF (bpp.EQ.8)   cext = '[byte]'
      IF (bpp.NE.8.AND.bpp.NE.16.AND.bpp.NE.32
     +    .AND.bpp.NE.-32.AND.bpp.NE.-64      ) THEN
         ctmp = 'unsupported data type '//cext(1:clen(cext))
         CALL error('rcfitsi',ctmp)
         ierr = 9
         RETURN
      ENDIF


C --- Now try and read the image line by line, casting the values stored
C     in the FITS array to native integer data type...
      pix1 = 1
      DO j=1,nrow
         IF (bpp.EQ.8) THEN
            CALL ftgpvb(STDIFIL,1,pix1,ncol,0,tmpbpx,anynull,ierr)
            IF (ierr.NE.0.AND.idbg.GT.1) CALL ftrprt ('STDOUT', ierr)
            IF (ierr.NE.0) GOTO 8
            DO i=1,ncol
               pix(i+(j-1)*ncol) = INT(tmpbpx(i))
            ENDDO
         ELSE IF (bpp.EQ.16) THEN
            CALL ftgpvi(STDIFIL,1,pix1,ncol,0,tmpipx,anynull,ierr)
            IF (ierr.NE.0.AND.idbg.GT.1) CALL ftrprt ('STDOUT', ierr)
            IF (ierr.NE.0) GOTO 8
            DO i=1,ncol
               pix(i+(j-1)*ncol) = INT(tmpipx(i))
            ENDDO
         ELSE IF (bpp.EQ.32) THEN
            CALL ftgpvj(STDIFIL,1,pix1,ncol,0,tmppix,anynull,ierr)
            IF (ierr.NE.0.AND.idbg.GT.1) CALL ftrprt ('STDOUT', ierr)
            IF (ierr.NE.0) GOTO 8
            DO i=1,ncol
               pix(i+(j-1)*ncol) = INT(tmppix(i))
            ENDDO
         ELSE IF (bpp.EQ.-32) THEN
            CALL ftgpve(STDIFIL,1,pix1,ncol,0,tmprpx,anynull,ierr)
            IF (ierr.NE.0.AND.idbg.GT.1) CALL ftrprt ('STDOUT', ierr)
            IF (ierr.NE.0) GOTO 8
            DO i=1,ncol
               pix(i+(j-1)*ncol) = INT(tmprpx(i))
            ENDDO
         ELSE IF (bpp.EQ.-64) THEN
            CALL ftgpvd(STDIFIL,1,pix1,ncol,0,tmpdpx,anynull,ierr)
            IF (ierr.NE.0.AND.idbg.GT.1) CALL ftrprt ('STDOUT', ierr)
            IF (ierr.NE.0) GOTO 8
            DO i=1,ncol
               pix(i+(j-1)*ncol) = INT(tmpdpx(i))
            ENDDO
         ENDIF
         pix1 = pix1 + ncol
      ENDDO


C --- Close the opened FITS or OIF file...
      CALL ftclos(STDIFIL,ierr)
      IF (ierr.NE.0.AND.idbg.GT.1) CALL ftrprt ('STDOUT', ierr)
      IF (ierr.NE.0) THEN
         ctmp = 'closing "'//filename(1:clen(filename))//'"'
         CALL error('rcfitsi',ctmp)
         ierr = 8
         RETURN
      ENDIF


C --- If in debugging mode, report some info upon exit...
      IF (idbg.GT.0) THEN
         CALL wsizstr (0, naxes, axlen, ctmp, i, ierr)
         IF (ierr.NE.0) THEN
            CALL error('rcfitsi','ierr > 0 on exit "wsizstr"')
            RETURN
         ENDIF
         j = clen(filename)
         ctmp(1:i+j) = filename(1:j)//ctmp(1:i)
         WRITE(STDOUT,'(A)') dbid//'read '//ctmp(1:i+j)//' into memory'
      ENDIF
C     Test whether the data is read correctly...
      IF (idbg.GT.1) THEN
         WRITE(STDOUT,'(A)') dbid//'verify the following data values:'
         axlen(2) = NINT(nrow/2.)
         j = NINT(ncol/2.)
         k = MIN(NINT(ncol/4.),100)
         m = clen(filename)
         DO i=(j-k),(j+k)
            axlen(1) = i
            CALL wsizstr (0, naxes, axlen, ctmp, n, ierr)
            ctmp(1:m+n) = filename(1:m)//ctmp(1:n)
            WRITE(STDOUT,*) dbid//ctmp(1:m+n)//' = ',
     +           pix(i+(axlen(2)-1)*ncol)
         ENDDO
      ENDIF

      RETURN


C --- Report error conditions from READ statement or from DO loop...
8     CALL cinit(ctmp)
      ctmp = 'while reading pixels (ftgpv*) from disk file'
      CALL error('rcfitsi','while reading pixels (ftgpv*)')
      ierr = 10
      RETURN

9     CALL cinit(ctmp)
      ctmp = 'reading image extension number from input string'
      CALL error('rcfitsi',ctmp)
      ierr = 1
      RETURN

      END


