C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      SUBROUTINE rjfgkvalq (crec, drval, dival, idbg, ierr)

C @(#)Subroutine: rjfgkvalq          Author: R.A. Jansen -- Oct 14, 2009
C @(#)
C @(#)  Purpose:  Read a complex  double precision-valued  keyword value
C @(#)    from a FITS header record  'crec',  already known to contain a
C @(#)    keyword value of this complex type,  and return its  real part
C @(#)    as 'rrval',  and its imaginary part as  'iival', both declared
C @(#)    as variables of the  REAL*8  data type.   For most host OS and
C @(#)    compiler implementations this corresponds to a 64-bit floating
C @(#)    point real.   If the keyword value  was written  with embedded
C @(#)    spaces,  'crec' will be modified  with the value written using
C @(#)    default formatting as a comma-separated pair of double precis-
C @(#)    ion values, enclosed within parentheses, and left-justified on
C @(#)    byte 11 in the header record.   If verbosity level 'idbg' > 0, 
C @(#)    additional messages may be printed.   Error status 'ierr' will
C @(#)    be non-zero upon exit, should any error be detected. Its value
C @(#)    (if >2000) uniquely identifies a known error condition.
C @(#)
C @(#)  Calling sequence:
C @(#)    CALL rjfgkvalq (crec, drval, dival, idbg, ierr),  where 'crec'
C @(#)    is a character string with a declared length of 80 characters,
C @(#)    and 'drval' and 'dival' are variables of the REAL*8 data type.
C @(#)    'idbg' and 'ierr' are both integers.
C @(#)
C @(#)  Note: this task is meant to be run after 'rjfvkval' (when 'ityp'
C @(#)    has the return value of 10) and 'rjfgkwrd'.
C @(#)  Note2: 64-bit floating point  REAL*8  values generally represent
C @(#)    at most  13 or 14 significant digits faithfully.  If even more
C @(#)    precision appears necessary, this task will warn but exit with
C @(#)    0 error status.
C @(#)
C @(#)  Dependencies:	clen, cinit, cidx, lenc, rjfvkmpq, rjfvk_pq
C @(#)
C @(#)  Revision history:
C @(#)    Oct 14, 2009 [RAJ] -- Initial version of this routine.
C @(#)------------------------------------------------------------------

      IMPLICIT NONE

C --- Declaration of input/return variables...
      CHARACTER	crec*80
      REAL*8	drval, dival
      INTEGER	idbg, ierr

C --- Declaration of local variables...
      CHARACTER	ctmp*80, cval*80, sval*80, dbid*19, cc
      REAL*8	dtmp, ddev
      INTEGER	clen, lenc, cidx, ic, ri, i, j, k, m, n, nd, np

C     'ip' and 'im' are shorthand integer equivalents in the ASCII col-
C     lating sequence of the plus and minus signs...
      INTEGER   ip, im

C     Flag 'lexp' is raised when an exponent character is found...
      LOGICAL	lexp

C     ------------------------------------------------------------------

C --- Return for non-zero input error status...
      IF (ierr.NE.0) RETURN

C --- Initialize the debugging ID string...
      dbid  = 'DEBUG(rjfgkvalq): '

C --- Initialize local variables...
      ip  = ICHAR('+')
      im  = ICHAR('-')

C --- Test that header record 'crec' indeed contains a complex-valued
C     keyword value and appears to have a valid format; then return as
C     character string 'ctmp' a space-separated copy of the real and
C     imaginary parts of that complex value (without the parentheses),
C     along with the byte positions in 'ctmp' of the first non-blank
C     character ('i'), the separating space ('j'; originally the comma),
C     and last non-blank character ('k') ...
      CALL rjfvkmpq (dbid,crec,ctmp,i,j,k,idbg,ierr)
      IF (ierr.NE.0) GOTO 9


C - - - - - - - -   Real part of complex keyword value   - - - - - - - -

C --- Copy the non-blank characters of the real part of the complex key-
C     word value from 'ctmp' to temporary value string 'cval', one char
C     at a time, while testing for any illegal character or combination
C     of characters that is inconsistent with a floating point value.
C     The 1st character in 'cval' will be blank to aid in list-directed
C     read...
      ri = 0
      CALL rjfvk_pq (dbid,ri,ctmp,i,j,cval,nd,lexp,idbg,ierr)
      IF (ierr.NE.0) GOTO 9

C --- Try to read the real floating point value from string 'cval'...
      dtmp = -9.9999999999D99
      m = clen(cval)
      IF (idbg.GT.2) WRITE(*,*) dbid//'  cval="'//cval(1:m)//'"'
      READ(cval,*,IOSTAT=ierr) dtmp
      IF (ierr.NE.0) THEN
         IF (idbg.GT.0) WRITE(*,*) dbid//'failed to read real part '//
     +      'of COMPLEX REAL*8 keyword value from record'
         ierr = 2771
         GOTO 9
      ENDIF
C     Return the real part of the complex keyword value (cast to REAL*8
C     data type)...
      drval = dtmp
      IF (idbg.GT.2) WRITE(*,'((A),E21.13e3,(A),E21.13e3)') ' '//dbid//
     +   '  dtmp=',dtmp,', drval=',drval

C --- Compare the number of digits in the mantissa to the estimated
C     precision required to faithfully represent the keyword value and
C     issue a warning, should we suffer significant loss of precision...
      np = 99
      ddev = DABS((dtmp-drval)/dtmp)
      IF (ddev.GT.0.0d0) np = ABS(INT(DLOG10(ddev)))
      IF (idbg.GT.2) WRITE(*,'((A),E21.13e3,(A),I2,(A),I2,(A))')
     +   ' '//dbid//'  ddev=',ddev,' (nd=',nd,',np=',np,')'
      i = clen(dbid) - 1
      n = clen(crec(1:8))
      IF (nd.GT.np) WRITE(*,*) 'WARNING'//dbid(6:i)//' value of '//
     +   crec(1:n)//' suffered loss of precision (O{10^',-(np+1),'})'

C --- If we temporarilly added "D0", strip it from 'cval' again; change
C     any explicit exponent character "D" (back) to "E"...
      IF (.NOT.lexp) THEN
         m = m - 2
         cval((m+1):(m+2)) = '  '
      ENDIF
      n = cidx(cval,'D')
      IF (n.NE.0) cval(n:n) = 'E'

C --- Store formatted contents of 'cval' for later use...
      CALL cinit(sval)
      sval(1:1) = '('
      cc = cval(2:2)
      ic = ICHAR(cc)
      IF (ic.EQ.ip .OR. ic.EQ.im) THEN
         sval(2:m) = cval(2:m)
      ELSE
         sval(2:2) = '+'
         IF (drval.LT.0) sval(2:2) = '-'
         sval(3:(m+1)) = cval(2:m)
         m = m + 1
      ENDIF
      sval((m+1):(m+1)) = ','


C - - - - - - - - Imaginary part of complex keyword value  - - - - - - -

C     Position of first and last non-blank char in the imaginary part...
      i = lenc(ctmp(j:80))
      i = i + j - 1
      k = clen(ctmp)

C --- Copy the non-blank characters of the imaginary part of the complex
C     keyword value from 'ctmp' to temporary value string 'cval', one at
C     a time, while testing for any illegal character or combination of
C     characters that is inconsistent with a floating point value.  The
C     first character in 'cval' will be blank to aid in list-directed
C     read...
      ri = 1
      CALL rjfvk_pq (dbid,ri,ctmp,i,k,cval,nd,lexp,idbg,ierr)
      IF (ierr.NE.0) GOTO 9

C --- Try to read the imaginary floating point value from 'cval'...
      dtmp = -9.9999999999D99
      m = clen(cval)
      IF (idbg.GT.2) WRITE(*,*) dbid//'  cval="'//cval(1:m)//'"'
      READ(cval,*,IOSTAT=ierr) dtmp
      IF (ierr.NE.0) THEN
         IF (idbg.GT.0) WRITE(*,*) dbid//'failed to read imaginary '//
     +      'part of COMPLEX REAL keyword value from record'
         ierr = 2772
         GOTO 9
      ENDIF
C     Return the imaginary part of the complex keyword value (cast to
C     REAL*8 data type)...
      dival = dtmp
      IF (idbg.GT.2) WRITE(*,'((A),E21.13e3,(A),E21.13e3)') ' '//dbid//
     +   '  dtmp=',dtmp,', dival=',dival

C --- Compare the number of digits in the mantissa to the estimated
C     precision required to faithfully represent the keyword value and
C     issue a warning, should we suffer significant loss of precision...
      np = 99
      ddev = DABS((dtmp-dival)/dtmp)
      IF (ddev.GT.0.0d0) np = ABS(INT(DLOG10(ddev)))
      IF (idbg.GT.2) WRITE(*,'((A),E21.13e3,(A),I2,(A),I2,(A))')
     +   ' '//dbid//'  ddev=',ddev,' (nd=',nd,',np=',np,')'
      i = clen(dbid) - 1
      n = clen(crec(1:8))
      IF (nd.GT.np) WRITE(*,*) 'WARNING'//dbid(6:i)//' value of '//
     +   crec(1:n)//' suffered loss of precision (O{10^',-(np+1),'})'

C --- If we temporarilly added "D0", strip it from 'cval' again; change
C     any explicit exponent character "D" (back) to "E"...
      IF (.NOT.lexp) THEN
         m = m - 2
         cval((m+1):(m+2)) = '  '
      ENDIF
      n = cidx(cval,'D')
      IF (n.NE.0) cval(n:n) = 'E'

C --- Append formatted contents of 'cval' to 'sval' for later use...
      i = clen(sval)
      j = i + 1
      k = i + m - 1
      cc = cval(2:2)
      ic = ICHAR(cc)
      IF (ic.EQ.ip .OR. ic.EQ.im) THEN
         sval(j:k) = cval(2:m)
      ELSE
         sval(j:j) = '+'
         IF (dival.LT.0) sval(j:j) = '-'
         sval((j+1):(k+1)) = cval(2:m)
         m = m + 1
      ENDIF
      sval((i+m):(i+m)) = ')'
      j = i + m


C - - - - - - - -  Return value of header record 'crec'  - - - - - - - -

C --- Return a possibly modified version of input header record 'crec',
C     with the keyword value (and comment) in fixed format...

      i = MAX(32,(10+j+2))
      k = cidx(crec((m+10):80),'/')
      k = k + (m+9)

C     First test whether the start position of any keyword comment needs
C     to be adjusted...
      IF (k.GT.i) THEN
C        Shift keyword comment to the left...
         crec(i:(i+(80-k))) = crec(k:80)
         DO n = (i+(80-k)+1),80,1
            crec(n:n) = ' '
         ENDDO
      ELSE IF (k.LT.i) THEN
C        Shift keyword comment to the right...
         crec(i:80) = crec(k:(80-(i-k)))
         DO n = k,(i-1),1
            crec(n:n) = ' '
         ENDDO
      ENDIF
C     Then left-justify the keyword value itself on byte 11 ...
      DO n = 11,(i-1),1
         crec(n:n) = ' '
      ENDDO
      crec(11:(10+j)) = sval(1:j)


9     IF (idbg.GT.2) WRITE(*,*) dbid//crec(1:clen(crec))
      IF (idbg.GT.1) WRITE(*,*) dbid//'on exit, drval=',drval,
     +   ', dival=',dival,', ierr=',ierr

      RETURN
      END


