C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      SUBROUTINE rjfvkval (crec, ityp, idbg, ierr)

C @(#)Subroutine: rjfvkval           Author: R.A. Jansen -- Sep 15, 2009
C @(#)
C @(#)  Purpose:  Determine the data type of the keyword value stored in
C @(#)    a 80-character FITS header record 'crec' and return it through
C @(#)    integer flag 'ityp'.  On exit, the value of 'ityp' encodes the
C @(#)    the keyword data-type as: -1 = undefined value or unrecognized
C @(#)    data type; 0 =comment record; 1 =character string; 2 =boolean;
C @(#)    3 =16/32-bit integer (INTEGER); 4 =32-bit integer (INTEGER*4);
C @(#)    5 =64-bit integer (INTEGER*8); 6=32-bit floating point (REAL);
C @(#)    7 =64-bit floating point (REAL*8); 8=16/32-bit complex integer
C @(#)    (INTEGER, INTEGER) ;  9 = 32-bit complex floating point (REAL,
C @(#)    REAL); 10 =64-bit complex floating point (REAL*8,REAL*8).   If
C @(#)    the data type was unrecognized,  rather than undefined, 'ierr'
C @(#)    will non-zero on exit. If verbosity level 'idbg'>0, additional
C @(#)    messages may be printed.  Should any error condition be detec-
C @(#)    ted, error flag 'ierr' will be non-zero upon exit.   Its value
C @(#)    (if >2000) uniquely identifies a known error condition.
C @(#)
C @(#)  Calling sequence:  CALL rjfvkval (crec, ityp, idbg, ierr), where
C @(#)    'crec' is a string with a declared length of 80 chars; 'ityp',
C @(#)    'idbg', and 'ierr' are all integers.
C @(#)
C @(#)  Note:  the estimate of the precision required to represent nume-
C @(#)    ric keyword values is a reasonable guess,  but can fail on the
C @(#)    safe side  when exponential notation was used to represent the
C @(#)    header keyword value (ie., it will return a data type encoding
C @(#)    of higher precision than was strictly necessary).  It does so,
C @(#)    because the estimated number of decimal digits doesn't account
C @(#)    for digits used in an exponent, if present.
C @(#)
C @(#)  Dependencies:  cinit, clen, lenc, cidx, ucase
C @(#)
C @(#)  Revision history:
C @(#)    Sep 15, 2009 [RAJ] -- Split off code from 'chkfits', 'exthdr',
C @(#)                          and 'getfval';  generalized it to a low-
C @(#)                          level routine  to be called prior to any
C @(#)                          of the 'rjfgkval*' family of routines.
C @(#)    Oct  1, 2009 [RAJ] -- Corrected error message for ierr=2502.
C @(#)    Oct  5, 2009 [RAJ] -- Allow recognition of non-compliant boo-
C @(#)                          lean-valued keyword values starting with
C @(#)                          a period; required a call to 'ucase'.
C @(#)------------------------------------------------------------------

      IMPLICIT NONE

C --- Declaration of input/return variables...
      CHARACTER crec*80
      INTEGER	ityp, idbg, ierr

C --- Declaration of local variables...
      CHARACTER	dbid*19, ctmp*80, cc
      INTEGER	clen, lenc, cidx
      INTEGER	ic, i, j, k, m, n, ii, nb, nc, nd
      LOGICAL	ldig, lndp

C --- The following variables are shorthand integer equivalents in the
C     ASCII collating sequence of the equals sign, space, quote, F, T,
C     opening parenthesis, digits 0 and 9, decimal point, +, -, and /,
C     respectively...
      INTEGER	ie, ib, iq, iFF, iTT, io, i0, i9, id, ip, im, is, iEE

C     ------------------------------------------------------------------

C --- Return for non-zero input error status...
      IF (ierr.NE.0) RETURN

C --- Initialize the debugging ID string...
      dbid = 'DEBUG(rjfvkval): '

C --- Initialize return variable...
      ityp = -1

C --- Initialize local variables...
      CALL cinit(ctmp)
      ie  = ICHAR('=')
      ib  = ICHAR(' ')
      iq  = 39
      iFF = ICHAR('F')
      iTT = ICHAR('T')
      io  = ICHAR('(')
      i0  = ICHAR('0')
      i9  = ICHAR('9')
      id  = ICHAR('.')
      ip  = ICHAR('+')
      im  = ICHAR('-')
      is  = ICHAR('/')
      iEE = ICHAR('E')
      ldig = .FALSE.
      lndp = .FALSE.

C --- Test whether byte 9 contains an equals sign or a blank...
      cc = crec(9:9)
      ic = ICHAR(cc)
      IF (ic.NE.ie) THEN
C     -- We are dealing with a comment record...
         IF (idbg.GT.1) WRITE(*,*) dbid//' record is a comment record'
         ityp = 0
      ELSE

C     -- Detected equals sign in byte 9; verify that byte 10 is blank...
         cc = crec(10:10)
         ic = ICHAR(cc)
         IF (ic.NE.ib) THEN
            IF (idbg.GT.0) WRITE(*,*) dbid//'non-blank character '//
     +         'follows "=" in byte 9'
            ierr = 2501
            GOTO 9
         ENDIF

C     -- We are likely dealing with a regular keyword=value pair; test
C        the first non-blank character in the value portion after con-
C        verting alphabetic characters to upper case...
         ctmp(1:71) = crec(10:80)
         CALL ucase(ctmp,ctmp)

C        The following counters store: i = actual length (position of
C        last non-blank character) of 'ctmp', j = position of the first
C        non-blank character, k = position of the first decimal point
C        or period (if present), m = position of the first forward slash
C        (if present), respectively ...
         i = clen(ctmp)
         j = lenc(ctmp)
         k = cidx(ctmp,'.')
         m = cidx(ctmp,'/')
         n = cidx(ctmp,',')

C        Count number of blank spaces between the start of the keyword
C        value and either the last non-blank char or the comment '/' and
C        add any exponent character (used to determine the required pre-
C        cision of numerical keyword values) ...
         nb = 0
         IF (m.EQ.0) THEN
            DO ii = j,i,1
               cc = ctmp(ii:ii)
               ic = ICHAR(cc)
               IF (ic.EQ.ib.OR.ic.EQ.iEE) nb = nb + 1
            ENDDO
         ELSE
            DO ii = j,(m-1),1
               cc = ctmp(ii:ii)
               ic = ICHAR(cc)
               IF (ic.EQ.ib.OR.ic.EQ.iEE) nb = nb + 1
            ENDDO
         ENDIF

C        First non-blank character to test...
         cc = ctmp(j:j)
         ic = ICHAR(cc)

C        Temporary booleans: 'ldig' is true if the first non-blank char
C        is a digit, 'lndp' if no decimal point or period is present...
         ldig = ( ic.GE.i0.AND.ic.LE.i9 )
         lndp = ( k.EQ.0 .OR. (m.GT.0.AND.k.GT.m) )

         IF (i.EQ.0.OR.ic.EQ.is) THEN
C        -- The value portion is blank: undefined keyword value...
            IF (idbg.GT.1) WRITE(*,*)dbid//' keyword value is undefined'
            ityp = -1

         ELSE IF (ic.EQ.iq) THEN
C        -- First character is a quote: char string valued keyword...
            IF (idbg.GT.1) WRITE(*,*) dbid//' keyword is string-valued'
            ityp = 1

         ELSE IF (ic.EQ.iFF.OR.ic.EQ.iTT) THEN
C        -- First character is T or F: boolean valued keyword...
            IF (idbg.GT.1) WRITE(*,*) dbid//' keyword is boolean-valued'
            ityp = 2

         ELSE IF (ic.EQ.id.AND.
     +            (ctmp(j+1:j+1).EQ.'F'.OR.ctmp(j+1:j+1).EQ.'T')) THEN
C        -- First character is a period, second is a T or F: keyword is
C           a non-compliant, but recognizable boolean valued keyword...
            IF (idbg.GT.1) WRITE(*,*) dbid//
     +         ' keyword is boolean-valued (but non-compliant)'
            ityp = 2

         ELSE IF ( (ic.EQ.ip.OR.ic.EQ.im.OR.ldig) .AND. lndp ) THEN
C        -- First character is +, -, or digit and there is no decimal
C           point present: integer valued keyword...

C           Estimate the precision required to represent the value...
            IF (m.EQ.0) THEN
               nc = i - j + 1
            ELSE
               nc = m - j - nb
            ENDIF
            IF (ic.EQ.ip.OR.ic.EQ.im) nc = nc - 1
            IF (nc.LE.4) THEN
C              We will require only 16-bit representation...
               IF (idbg.GT.1) WRITE(*,*) dbid//
     +            ' keyword is integer-valued'
               ityp = 3
            ELSE IF (nc.LE.9) THEN
C              We will require only 32-bit representation...
               IF (idbg.GT.1) WRITE(*,*) dbid//
     +            ' keyword is integer*4-valued'
               ityp = 4
            ELSE
C              We will likely require 64-bit representation...
               IF (idbg.GT.1) WRITE(*,*) dbid//
     +            ' keyword is integer*8-valued'
               ityp = 5
            ENDIF

         ELSE IF ( (ic.EQ.ip.OR.ic.EQ.im.OR.ic.EQ.id.OR.ldig) ) THEN
C        -- First character is +, -, digit, or decimal point and there
C           is a decimal point present: floating point valued keyword...

C           Estimate the precision required to represent the value...
            IF (m.EQ.0) THEN
               nc = i - j + 1
               nd = i - k
            ELSE
               nc = m - j - nb
               nd = m - k - nb - 1
            ENDIF
            IF (ic.EQ.ip.OR.ic.EQ.im) nc = nc - 1
            IF (nc.LE.10.AND.nd.LE.7) THEN
C              We will require only 32-bit representation...
               IF (idbg.GT.1) WRITE(*,*) dbid//' keyword is real-valued'
               ityp = 6
            ELSE
C              We will likely require 64-bit representation...
               IF (idbg.GT.1) WRITE(*,*) dbid//
     +            ' keyword is real*8-valued'
               ityp = 7
            ENDIF

         ELSE IF (ic.EQ.io.AND.n.GT.0) THEN
C        -- First character is an opening parenthesis: complex valued
C           keyword...
            j = j + 1

C           Count the number of embedded blank spaces within the real
C           part of the complex value...
            nb = 0
            DO ii = j,(n-1),1
               cc = ctmp(ii:ii)
               ic = ICHAR(cc)
               IF (ic.EQ.ib) nb = nb + 1
            ENDDO

            cc = ctmp(j:j)
            ic = ICHAR(cc)

            IF (k.EQ.0.OR.k.GT.n) THEN
C              Real part does not contain decimal point: integer
C              Estimate the type and/or precision required to represent
C              the real part of the complex value...
               nc = n - j - nb
               IF (ic.EQ.ip.OR.ic.EQ.im) nc = nc - 1
               IF (nc.LE.9) THEN
C                 We will likely require 32-bit representation...
                  IF (idbg.GT.1) WRITE(*,*) dbid//
     +               ' real part of complex keyword is integer-valued'
                  ityp = 8
               ELSE
C                 We will likely require 64-bit representation...
                  IF (idbg.GT.1) WRITE(*,*) dbid//
     +               ' real part of complex keyword is integer*8-valued'
                  ityp = 10
               ENDIF
            ELSE
C              Real part contains a decimal point: floating point
C              Estimate the type and/or precision required to represent
C              the real part of the complex value...
               nc = n - j - nb
               nd = n - k - nb - 1
               IF (ic.EQ.ip.OR.ic.EQ.im) nc = nc - 1
               IF (nc.LE.10.AND.nb.LE.7) THEN
C                 We will likely require 32-bit representation...
                  IF (idbg.GT.1) WRITE(*,*) dbid//
     +               ' real part of complex keyword is real-valued'
                  ityp = 9
               ELSE
C                 We will likely require 64-bit representation...
                  IF (idbg.GT.1) WRITE(*,*) dbid//
     +               ' real part of complex keyword is real*8-valued'
                  ityp = 10
               ENDIF
            ENDIF

C           We will assume that the precision required for the real part
C           of the complex keyword value is indicative for that of the
C           imaginary part as well...

         ELSE
C           Unrecognized data type (possibly malformatted header)...
            IF (idbg.GT.0) WRITE(*,*) dbid//'malformatted header '//
     +         'record: unrecognized keyword data type'
            ierr = 2502
            GOTO 9

         ENDIF

      ENDIF


9     IF (idbg.GT.2) WRITE(*,*) dbid//crec(1:clen(crec))
      IF (idbg.GT.1) WRITE(*,*) dbid//'on exit, ityp=',ityp,', ierr=',
     +   ierr

      RETURN
      END


